<?php

namespace Drupal\zohomail\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Response;
use Drupal\zohomail\Helper\ZohoMailApi;
use Drupal\zohomail\Helper\ZConstants;
use Drupal\Core\Access\CsrfTokenGenerator;
use \Drupal\Core\Url;

class ZohoMailController extends ControllerBase {

  /**
   * Render the admin page with a Twig template and custom JavaScript.
   */
  public function adminPage() {
    // Data to pass to the Twig template.
    global $base_url;
    $config = \Drupal::config('zohomail.settings');
    $csrf_service = \Drupal::service('csrf_token');
    $csrf_token = $csrf_service->get('zohomail_config_form');
    $is_account_configured = false;
    $is_mail_configured = false;
    $emailIdList = [];
    $callBackURL = ZConstants::getCallBackURL();
    


    // Get specific configuration values.
    $account_id = $config->get('account_id');
    if(!empty($account_id)){
      $domain = $config->get('domain');
      $clientId = base64_decode($config->get('client_id'));
      $clientSecret = base64_decode($config->get('client_secret'));
      $from_address =  $config->get('from_address');
      $zmailAPI = new ZohoMailApi($config);
      $emailDetail = $zmailAPI->getZohoMailAccountDetails();
      $sendEmailList = $emailDetail['sendmail_details'];
      $is_account_configured = true;
      if(isset($sendEmailList) && in_array($from_address,$sendEmailList)){
        $is_mail_configured=true;
      }
      $from_address = $from_address;
      $from_name = $config->get('from_name');
    } 
    else {
      $domain = 'zoho.com';
      $clientId = '';
      $clientSecret = '';
      $from_address = '';
      $from_name = '';
      $emailaddress=[];
      $sendEmailList=[];
    }
    
    $data = [
      'domain' => $domain,
      'client_id' => $clientId,
      'client_secret' => $clientSecret,
      'from_address' => $from_address,
      'from_name' => $from_name,
      'csrf_token' => $csrf_token,
      'is_account_configured' => $is_account_configured,
      'is_mail_configured' => $is_mail_configured,
      'emailIdList' => $sendEmailList,
      'callBackURL' => $callBackURL
      
    ];

    return [
      '#theme' => 'zohomail_admin_page_js',
      '#data' => $data,
      '#csrftoken' => $csrf_token,
      '#attached' => [
        'library' => [
          'zohomail/admin_page_js',
        ],
      ],
    ];
  }
  public function saveEmailConfiguration() {
    $request = \Drupal::request()->request;
    $zohomail_csrftoken = $request->get('zohomail_csrftoken');
    $from_address =  $request->get('from_address');
    $config = \Drupal::config('zohomail.settings');
    $respObj = json_decode("{}",true);

    $zmailAPI = new ZohoMailApi($config);
    $emailDetail = $zmailAPI->getZohoMailAccountDetails();
    $sendEmailList = $emailDetail['sendmail_details'];
    if(isset($sendEmailList) && in_array($from_address,$sendEmailList)) {
      $account_id = $emailDetail['account_id'];
      $mailRes = $this->sendTestMail($from_address,$zmailAPI,$account_id);

      $config = \Drupal::configFactory()->getEditable('zohomail.settings');
      $config->set('account_id',$account_id)
              ->set('from_address',$from_address)
              ->set('allowed_emails',$sendEmailList)
              ->save();
      $respObj["result"] = "success";
    } else {
      $respObj["result"] = "error";
    }
    
   
    return new JsonResponse($respObj);
    
  }
  
  public function testEmailConfiguration() {
    
    $config = \Drupal::config('zohomail.settings');
    $respObj = json_decode("{}",true);
    
    $request = \Drupal::request()->request;
    $csrf_service = \Drupal::service('csrf_token');
    $zohomail_csrftoken = $request->get('zohomail_csrftoken');
    $from_address = $config->get('from_address');
    $account_id =$config->get('account_id');
    /*
    if (!$csrf_service->validate($zohomail_csrftoken, 'zohomail_config_form')) {
      $respObj["result"] = "failure";
      $respObj['error_message'] = "Invalid request";
    }
      */

    $zmailAPI = new ZohoMailApi($config);
    $mailRes = $this->sendTestMail($from_address,$zmailAPI,$account_id);
    
    if($mailRes->result !== "success") {
      $errorCode = $mailRes->details->code;
      $respObj["result"] = "failure";
      if($errorCode === 'SERR_157'){
        $respObj['error_message'] = "Enter valid send mail token to complete configuration";
      } else{
        $respObj['error_message'] = $mailRes->details->message;
      }
    } 
    else {
      $respObj["result"] = "success";
    }
      
  
      return new JsonResponse($respObj);
  }
  public function authorizeZohoMail() {
    global $base_url;
    $request = \Drupal::request()->request;
    $csrf_service = \Drupal::service('csrf_token');
    $zohomail_csrftoken = $request->get('zohomail_csrftoken');
    
    $domain = $request->get('domain');
    $clientId = $request->get('clientId');
    $clientSecret = $request->get('clientSecret');
    


    
    $respObj = json_decode("{}",true);
    $request = \Drupal::request()->request;
    if (!$csrf_service->validate($zohomail_csrftoken, 'zohomail_config_form')) {
      $respObj["result"] = "failure";
      $respObj['error_message'] = "Invalid request";
      return $this->writeJSONResponse($respObj);
    }

    $callBackURL = ZConstants::getCallBackURL(); 
    $state=base64_encode($clientId."::".$clientSecret."::".$domain."::".$zohomail_csrftoken);
    $respObj["authorize_url"] = "https://accounts.".ZConstants::$domains[$domain]."/oauth/v2/auth?client_id=".$clientId."&response_type=code&access_type=offline&prompt=consent&redirect_uri=".$callBackURL."&state=".$state."&scope=".ZConstants::mail_scopes;
    $respObj["result"] = "success";
    return $this->writeJSONResponse($respObj);
  }
  public function callBack() {
    global $base_url;
    $request = \Drupal::request()->request;
    $csrf_service = \Drupal::service('csrf_token');
    
    $config = \Drupal::config('zohomail.settings');

    $response = new Response(
      "Content",
      Response::HTTP_OK,
      ['content-type' => 'text/html']
  );
    try {
      $authCode = $_GET['code'];
      $encoded_state = $_GET['state'];
       
      if (!$authCode) {
         // throw new LocalizedException(__('Authorization code missing.'));
      }
      $state=explode("::",base64_decode($encoded_state));
      $clientId = $state[0];
      $clientSecret = $state[1];
      $domain = $state[2];
      $form_key = $state[3];
     
      $completeRedirectUrl = ZConstants::getCallBackURL();
      $url = "https://accounts.".ZConstants::$domains[$domain]."/oauth/v2/token?code=".$authCode."&client_id=".$clientId."&client_secret=".$clientSecret."&redirect_uri=".$completeRedirectUrl."&grant_type=authorization_code&state=".$encoded_state;
      $curl = curl_init();
      curl_setopt_array($curl, array(
  CURLOPT_URL => $url,
  CURLOPT_RETURNTRANSFER => true,
  CURLOPT_ENCODING => "",
  CURLOPT_MAXREDIRS => 10,
  CURLOPT_TIMEOUT => 30,
  CURLOPT_SSLVERSION => CURL_SSLVERSION_TLSv1_2,
  CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
  CURLOPT_CUSTOMREQUEST => "POST"
  ));
      $response = curl_exec($curl);
      $err = curl_error($curl);
      curl_close($curl);
      $respObj = json_decode("{}",true);
      $httpcode = curl_getinfo($curl, CURLINFO_HTTP_CODE);
      if($httpcode == '200') {
          $body = json_decode($response,true);
          if(array_key_exists('error',$body)){
              $respObj["result"] = "error";
              $respObj["message"] = json_decode($response);
          } else {
             $config = \Drupal::configFactory()->getEditable('zohomail.settings');
                $config->set('domain', $domain)
                    ->set('client_id',base64_encode($clientId))
                    ->set('client_secret',base64_encode($clientSecret))
                    ->set('refresh_token',base64_encode($body['refresh_token']))
                    ->set('access_token',base64_encode($body['access_token']))
                    ->set('timestamp',time())
                    ->save();
                $zmailApi = new ZohoMailApi($config);
                $emailDetail = $zmailApi->getZohoMailAccountDetails(); 
                $accountId = $emailDetail["account_id"];
                $config->set('account_id', $accountId)->save();
                $respObj["result"] = "success";
          }
          
          
      }else{
          $respObj["result"] = "error";
          $respObj["message"] = json_decode($response);
      }
      echo "<script>window.opener.postMessage(".json_encode($respObj).");window.close();</script>";
      

      return;
  } catch (\Exception $e) {
     
  }
    return $this->writeJSONResponse($respObj);
  }
  private function sendTestMail($fromAddress,$zMailApi,$account_id)
    {
      $mail_data = array();
      $from = json_decode('{}');
      $from->address = $fromAddress;
      $emailDetail = array();
      $toArray = array();
      
      $mail_data['fromAddress'] = $fromAddress;
      $mail_data['toAddress'] = $fromAddress;
  
      $mail_data['subject'] = "This is a test email for the Dupal";
      $mail_data['content'] = "This is a test email for the Drupal";
      
      
      return $zMailApi->sendTestMail($mail_data,$account_id);
        
    }
    private function writeJSONResponse($jsonMessage) {
      return new JsonResponse($jsonMessage);
    }
}
